//Authors: Ates Akaydn, M.Bugra Yildiz
// Phong Globe Shader
// Date: 24.10.2011

//#version 140

//Input
in vec4 vPosition; //Position of the vertex
in vec4 vNormal; //Normal vector of the vertex
in vec4 vTexCoord; //Texture coordinates of the vertex

uniform float rotation; //Rotation around y-axis in degrees
uniform vec4 lightPosition; //Position of the scene light

//Output
varying vec3 iPos,iNormal, iEyeDir, iReflectDir, iLightDir, iTexCoord; //Interpolated output normal, eye direction and light direction vectors

void main()
{

	//This rotations the globe around it self by a set amount(rotation is controlled by the application) around y-axis
	float rotRad = radians(rotation);
	mat4 rotMatrix = mat4( cos( rotRad ), 0.0, -sin( rotRad ), 0.0,
									 0.0, 1.0,            0.0, 0.0,
						   sin( rotRad ), 0.0,  cos( rotRad ), 0.0,
									 0.0, 0.0,	          0.0, 1.0 );	

	//Inverse transform normals. And carry out perspective correct interpolationg for fragment shader
	iNormal = gl_NormalMatrix * mat3(rotMatrix) * vNormal.xyz;
	iTexCoord = vTexCoord.xyz;
	iPos = vPosition.xyz;
	
	//Calculate vertex coordinates in eye space...
	mat4 mvMatrix = gl_ModelViewMatrix * rotMatrix;
	
	vec4 mvVertex = mvMatrix * vPosition;
	
	//Calculate light coordinates in eye space...
	vec4 mvLightPosition = gl_ModelViewMatrix * lightPosition;
	
	//Eye direction should be the negated vertex position in eye space
	iEyeDir = -mvVertex.xyz;
	
	//This is view space direction... cube map exists in world space
	iReflectDir = reflect(-iEyeDir, iNormal);
	iReflectDir = inverse(gl_NormalMatrix) * iReflectDir;
	
	//Light direction should be the difference vector in between vertex and light position vectors
	//in eye space
	iLightDir = mvLightPosition.xyz - mvVertex.xyz;
	
	gl_Position = gl_ProjectionMatrix * mvVertex;
}