//Authors: Ates Akaydn, M.Bugra Yildiz
// Phong Shader
// Date: 26.11.2011

//#version 140

//INPUT

//Texture Parameters
uniform sampler2D diffuseTexture; //Diffuse Texture
uniform sampler2D specularTexture; //Specular Map
uniform sampler2D ambientTexture; //Ambient Map
uniform float tiling = float(1.0); //Amount of Tiling for all textures

//Light Properties
//TODO: Support for multiple lights will be added in the near future
uniform vec4 lightColor; //Color of the scene light
uniform vec4 lightPosition; //Position of the scene light

//TODO: Light attenuation parameters will be controllable from the application in the near future
uniform float lightAttRange = float(250.0); //Range of light attenuation... where it perishes
uniform float lightAttExp = float(2.0); //Light attenuation exponent... controls how rapidly light intensity drops

//Material Parameters
uniform vec4 matAmbientCoef = vec4( 0.01, 0.01, 0.01, 1.0 );
uniform vec4 matDiffuseCoef = vec4( 1.0, 1.0, 1.0, 1.0 );
uniform vec4 matSpecularCoef = vec4( 1.0, 1.0, 1.0, 1.0 );
uniform float matShininess = 8.0; 

//Interpolated parameters from vertex shader
varying vec3 iPos, iNormal, iEyeDir, iLightDir, iTexCoord; //Interpolated output normal, eye direction and light direction vectors

//Main routine
void main()
{
	vec4 ambientTexColor = texture(ambientTexture, tiling * iTexCoord.xy);
	vec4 diffuseTexColor = texture(diffuseTexture, tiling * iTexCoord.xy);
	vec4 specularTexColor = texture(specularTexture, tiling * iTexCoord.xy);
		
	vec4 final_color = matAmbientCoef * ambientTexColor;
	vec3 N = normalize(iNormal);
    vec3 L = normalize(iLightDir);

    float lambertTerm = dot(N,L);
    if (lambertTerm > 0.0)
    {
		float r = length( lightPosition.xyz - iPos );
		float k = 1 / pow(lightAttRange,2.0);
		float I = length(lightColor);
		
		float lightAtt = clamp( I / (1 + k * pow( r, lightAttExp ) ),0.0,1.0 );
		    
		final_color.rgb += (lightColor.rgb * matDiffuseCoef.rgb * diffuseTexColor.rgb *lambertTerm ) * lightAtt;
		vec3 E = normalize(iEyeDir);
		vec3 R = reflect(-L, N);
		float specular = pow(max(dot(R, E), 0.0), matShininess);
		final_color.rgb += (lightColor.rgb * matSpecularCoef.rgb * specularTexColor.rgb * specular) * lightAtt;
    }
	final_color.a = matDiffuseCoef.a;    
	gl_FragColor = clamp(final_color, 0.0, 1.0);	

}