//Authors: Ates Akaydn, M.Bugra Yildiz
// Phong SpotLight Shader
// Date: 26.11.2011

//#version 140

//Input
in vec4 vPosition; //Position of the vertex
in vec4 vNormal; //Normal vector of the vertex
in vec4 vTexCoord; //Texture coordinates of the vertex

uniform vec4 lightPosition; //Position of the scene light

//Output
varying vec3 iPos, iNormal, iEyeDir, iLightDir, iTexCoord; //Interpolated output normal, eye direction and light direction vectors

void main()
{	
	//Inverse transform normals. And carry out perspective correct interpolationg for fragment shader
	iNormal = gl_NormalMatrix * vNormal.xyz;
	iTexCoord = vTexCoord.xyz;
	iPos = vPosition.xyz;
	
	//Calculate vertex coordinates in eye space...
	vec4 mvVertex = gl_ModelViewMatrix * vPosition;
	
	//Calculate light coordinates in eye space...
	vec4 mvLightPosition = gl_ModelViewMatrix * lightPosition;
	
	//Eye direction should be the negated vertex position in eye space
	iEyeDir = -mvVertex.xyz;
	
	//Light direction should be the difference vector in between vertex and light position vectors
	//in eye space
	iLightDir = mvLightPosition.xyz - mvVertex.xyz;
	
	gl_Position = gl_ProjectionMatrix * mvVertex;
}