//Authors: Ates Akaydn, M.Bugra Yildiz
// Phong Shader with Normal Mapping
// Date: 26.11.2011

//#version 140

//Input
in vec4 vPosition; //Position of the vertex
in vec4 vTexCoord; //Texture coordinates of the vertex

const float pi = 3.14159; //Approximation for pi
uniform float time; //Current time
uniform float speed1 = float(6.4), wavelength1 = float(32.0), amplitude1 = float(0.7);
uniform float speed2 = float(9.6), wavelength2 = float(56.0), amplitude2 = float(0.8);
uniform float speed3 = float(4.0), wavelength3 = float(36.0), amplitude3 = float(0.9);
uniform vec2 wavedir1 = vec2(0.5,0.5), wavedir2 = vec2(1.0,0.0), wavedir3 = vec2(0.0,-1.0); 

uniform vec4 lightPosition; //Position of the scene light

//Output
varying vec3 iPos, wEyeDir, iEyeDir, iLightDir, iTexCoord; //Interpolated output normal, eye direction and light direction vectors

float wave(float speed, float wavelength, float amplitude, vec2 direction, float x, float y) {
    float frequency = 2*pi/wavelength;
    float phase = speed * frequency;
    float theta = dot(direction, vec2(x, y));
    return amplitude * sin(theta * frequency + time * phase);
}

float waveHeight(float x, float y) {
    float height = 0.0;
    height += wave(speed1, wavelength1, amplitude1, wavedir1, x, y);
    height += wave(speed2, wavelength2, amplitude2, wavedir2, x, y);
    height += wave(speed3, wavelength3, amplitude3, wavedir3, x, y);
    return height;
}

void main()
{	
	//Inverse transform normals. And carry out perspective correct interpolationg for fragment shader
	iTexCoord = vTexCoord.xyz;
	
	//Add up water computed height
	vec4 wPosition = vPosition;
	wPosition.y += waveHeight( wPosition.x, wPosition.z );
	iPos = wPosition.xyz;
	
	//Calculate vertex coordinates in eye space...
	vec4 mvVertex = gl_ModelViewMatrix * wPosition;
	
	//Calculate light coordinates in eye space...
	vec4 mvLightPosition = gl_ModelViewMatrix * lightPosition;
	
	//Eye direction should be the negated vertex position in eye space
	iEyeDir = -mvVertex.xyz;
	wEyeDir = inverse(gl_NormalMatrix)*iEyeDir; 
	
	//Light direction should be the difference vector in between vertex and light position vectors
	//in eye space
	iLightDir = mvLightPosition.xyz - mvVertex.xyz;
	
	gl_Position = gl_ProjectionMatrix * mvVertex;
	
}